<?php 
require_once("icwfData.inc.php");
require_once("FormValidation.inc.php");

class EventOption implements icwfData {
  private static $noExport = array();
  private static $notInArray = array("validation", "delete");
  private $validation;
  
  private $event_option_id;
  private $event_option_group_id;
  private $event_option;
  private $person_limit;
  private $booking_limit;
  private $option_type;
  private $list;
  private $eventFees;
  
  private $no_of_bookings;
  private $delete;

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  private function initialize() {
    foreach ($this as $key => $value)
      if($key == "eventFees")
        $this->$key = array();
      else
        $this->$key = null;
  }
  
  public function __construct() {
    $this->initialize();
    
    $this->delete = false;
    
    $this->validation = new FormValidation(array(
        "option"=>array("required"=>true)
      ),
      array(
        "option"=>_("Please enter a event option name.")
      ));
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // get values
  public function __get($name) {
    if($name == "id") $name = "event_option_id";
    if (array_key_exists($name, get_class_vars(get_class())) && !in_array($name, self::$noExport))
      return $this->$name;
      
    return null;
  }
  
  public function getCurrentFee($first=false) {
    if($first) return reset($this->eventFees);
    
    foreach($this->eventFees as $feeCount => $fee) {
      if (!$fee->deadline || $fee->deadline >= date("Y-m-d")) {
        return $fee;
      }
    }
    
    return null;
  }

  public function getLastDeadline() {
    return end($this->eventFees)->deadline;
  }

  public function getIDName() {
    return "event_option_id";
  }
  
  public function toArray() {
    $array = array();
    foreach ($this as $key => $value)
      if(!in_array($key, self::$notInArray)) {
        if($key == "eventFees")
          foreach($this->eventFees as $feeKey => $fee)
            $fee->toArray();
        else
          $array[$key] = $value; 
      }
    return $array;
  }

  public function getPostBackData() {
    $postBackArray = array($this->getIDName()=>$this->id);
    foreach ($this->eventFees as $feeId => $fee) {
      $tmp = $fee->getPostBackData();
      foreach ($tmp as $key => $value)
        $postBackArray["{$key}[{$feeId}]"] = $value; 
    }
    
    return $postBackArray;
  }
  
  public function feeExists($feeId) {
    foreach ($this->eventFees as $feeKey => $fee)
      if($fee->id == $feeId) return true;

    return false;
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////////
  // set values
  public function setValidation(FormValidation $validation) {
    $this->validation = $validation;
  }

  public function deleteOption($delete) {
    $this->delete = ($delete ? 1 : 0);
  }
  
  public function setGroupId($groupId) {
    $this->event_option_group_id = $groupId;
  }
  
  public function fromArray($array) {
    return true;
  }
  
  public function fromPOSTArray($array, $groupIdx, $optionIdx) {
    foreach ($this as $key => $value) {
      switch ($key) {
        case "validation":
        case "delete":
          break;
        
        case "event_option_group_id":
        case "event_option_id":
        case "person_limit":
        case "booking_limit":
        case "option_type":
          if(isset($array[$key][$groupIdx][$optionIdx]) && preg_match("~^[0-9]+$~", $array[$key][$groupIdx][$optionIdx]))
            $this->$key = $array[$key][$groupIdx][$optionIdx];
          else
            $this->$key = null;
          break;
        case "event_option":
          if(isset($array[$key][$groupIdx][$optionIdx]))
            $this->$key = $array[$key][$groupIdx][$optionIdx];
          else
            $this->$key = null;
          break;
        case "list":
          if(isset($array[$key][$groupIdx][$optionIdx]))
            $this->$key = KDbTools::trimArray(explode(",", $array[$key][$groupIdx][$optionIdx]));
          else
            $this->$key = null;
          break;
          
        case "eventFees":
          foreach ($this->eventFees as &$fee) {
            if(!isset($array['event_fee_id'][$groupIdx][$optionIdx]) || 
               !in_array($fee->event_fee_id, $array['event_fee_id'][$groupIdx][$optionIdx])) {
              $fee->deleteFee(true);
            }
          }
          if(isset($array['event_fee'][$groupIdx][$optionIdx])) {
            foreach ($array['event_fee'][$groupIdx][$optionIdx] as $feeIdx => $feeValue) {
              if($array['event_fee_id'][$groupIdx][$optionIdx][$feeIdx]) {
                $this->eventFees[$array['event_fee_id'][$groupIdx][$optionIdx][$feeIdx]]->fromPOSTArray($array, $groupIdx, $optionIdx, $feeIdx);
              }
              else {
                $newFee = new EventFee();
                $newFee->fromPOSTArray($array, $groupIdx, $optionIdx, $feeIdx);
                $this->eventFees[] = $newFee;
              }
            }
          }
          break;
          
        default:
          break;
      }
    }
    return true;
    //return $this->validation->validate($array);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - read
  public function readData(DbConnection $DB, $conditions) {
    $whereConditions = array();
    foreach ($conditions as $key => $value)
      if(array_key_exists($key, get_class_vars(get_class())))
        $whereConditions[] = "eo.$key=$value";

    $result = $DB->query("SELECT eo.*, sum(if(ub.event_fee_id, 1, 0)*ub.value_int) no_of_bookings
                          FROM event_options eo
                            LEFT JOIN user_booking_options ubo ON (ub.event_option_id=eo.event_option_id AND ubo.deleted=0)
                          WHERE ".implode(" AND ", $whereConditions)."
                          GROUP BY eo.event_option_id;");
    if(!$DB->errno() && $DB->num_rows($result)>0) {
      $option = $DB->fetch_array($result);
      foreach ($option as $property => $value) {
        if(property_exists($this, $property)) {
          if($property=="list")
            $this->$property = explode(",", $value);
          else
            $this->$property = $value;
        }
      }
      $result = $DB->query("SELECT event_fee_id 
                            FROM event_fees 
                            WHERE event_option_id='{$this->event_option_id}'
                            ORDER BY event_fee_id;");
      $this->eventFees = array();
      while ($feeId = $DB->fetch_array($result)) {
        $fee = new EventFee();
        $fee->readData($DB, array("event_fee_id"=>$feeId['event_fee_id']));
        $this->eventFees[$feeId['event_fee_id']] = $fee;
      }
      
      return array(0, "");
    }
    else
      return array($DB->errno(), $DB->error());
  }
  
  static public function readList(DbConnection $DB) {
    return array();
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - write
  public function writeData(DbConnection $DB) {
    if($this->delete)
      return self::deleteData($DB, $this->id);
    else if(isset($this->event_option_id) && $this->event_option_id) {
      $result = $DB->query("UPDATE event_options SET
          event_option_group_id='{$this->event_option_group_id}',
          event_option='{$this->event_option}',
          person_limit=".(!$this->person_limit ? "'0'" : "'{$this->person_limit}'").",
          booking_limit=".(!$this->booking_limit ? "'1'" : "'{$this->booking_limit}'").",
          option_type=".(!$this->option_type ? "null" : "'{$this->option_type}'").",
          list=".(!in_array($this->option_type, array(EVENT_OPTION_SELECT)) ? "null" : "'".implode(",", $this->list)."'")."
        WHERE event_option_id='{$this->event_option_id}';");
    }
    else {
      $result = $DB->query("INSERT INTO event_options(
          event_option_group_id,
          event_option,
          person_limit,
          booking_limit,
          option_type,
          list
        ) VALUES(
          '{$this->event_option_group_id}',
          '{$this->event_option}',
          ".(!$this->person_limit ? "'0'" : "'{$this->person_limit}'").",
          ".(!$this->booking_limit ? "'1'" : "'{$this->booking_limit}'").",
          ".(!$this->option_type ? "null" : "'{$this->option_type}'").",
          ".(!in_array($this->option_type, array(EVENT_OPTION_SELECT)) ? "null" : "'".implode(",", $this->list)."'")."
        );");
      
      $this->event_option_id = $DB->insert_id();
    }
    foreach ($this->eventFees as &$fee) {
      $fee->setOptionId($this->event_option_id);
      $fee->writeData($DB);
    }
    return array($DB->errno(), $DB->error());
  }

  static public function deleteData(DbConnection $DB, $id) {
    if(preg_match("~^[0-9]+$~", $id))
      $condition = "event_option_id='{$id}'";
    else
      $condition = $id;
      
    $optionIds = $DB->result($DB->query("SELECT group_concat(event_option_id SEPARATOR ', ') 
                                          FROM event_options 
                                          WHERE $condition;"), 0, 0);
    $result = $DB->query("DELETE FROM event_options WHERE {$condition};");
    
    if(!$DB->errno() && $DB->affected_rows()>0) {
      UserBookingOption::deleteData($DB, "event_option_id IN ({$optionIds})");
      EventFee::deleteData($DB, "event_option_id IN ({$optionIds})");
      return array(0, "");
    }
    else 
      return array($DB->errno(), $DB->error());
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - misc
  static public function createTable(DbConnection $DB) {
    $result = $DB->query("
      CREATE TABLE event_options (
        event_option_id int unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY,
        event_option_group_id int unsigned,
        event_option text,
        person_limit mediumint unsigned,
        booking_limit mediumint unsigned,
        option_type tinyint unsigned,
        list text,
        INDEX event_option_group_id (event_option_group_id)
      );");
    if ($result) { echo _("&nbsp;&nbsp;- creating table").": 'event_options'.<br>\n"; }
    else { echo _("&nbsp;&nbsp;<b>- Error when creating table</b>")." 'event_options'<br>\n"; $error = 1; }
  }  
}
?>