<?php 
require_once("icwfData.inc.php");
require_once("FormValidation.inc.php");

class EventOptionsGroup implements icwfData {
  private static $noExport = array();
  private static $notInArray = array("validation", "delete");
  private $validation;
  
  private $event_option_group_id;
  private $event_id;
  private $group_name;
  private $group_type;
  private $exclusive;
  private $allow_changes;
  private $enable_waitlist;
  private $required;
  private $description;
  private $eventOptions;
  
  private $delete;

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  private function initialize() {
    foreach ($this as $key => $value)
      if($key == "eventOptions")
        $this->$key = array();
      else
        $this->$key = null;
  }
  
  public function __construct() {
    $this->initialize();
    
    $this->delete = false;
    
    $this->validation = new FormValidation(array(
        "group_name"=>array("required"=>true),
        "group_type"=>array("required"=>true)
      ),
      array(
        "group_name"=>_("Please enter a group name."),
        "group_type"=>_("Please choose a group type.")
      ));
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // get values
  public function __get($name) {
    if($name == "id") $name = "event_option_group_id";
    if (array_key_exists($name, get_class_vars(get_class())) && !in_array($name, self::$noExport))
      return $this->$name;
      
    return null;
  }

  public function getIDName() {
    return "event_option_group_id";
  }
  
  public function toArray() {
    $array = array();
    foreach ($this as $key => $value)
      if ($key=="eventOptions")
        $array[$key] = $value->toArray();
      else if(!in_array($key, self::$notInArray))
        $array[$key] = $value; 
    return $array;
  }
  
  public function getPostBackData() {
    $postBackArray = array($this->getIDName()=>$this->id);
    foreach ($this->eventOptions as $optionId => $option) {
      $tmp = $option->getPostBackData();
      foreach ($tmp as $key => $value)
        $postBackArray[preg_replace('~^([^\[]+)(.*)$~', "$1[{$optionId}]$2", $key)] = $value; 
    }
    
    return $postBackArray;
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////////
  // set values
  public function setValidation(FormValidation $validation) {
    $this->validation = $validation;
  }
  
  public function deleteGroup($delete) {
    $this->delete = ($delete ? 1 : 0);
  }
  
  public function setEventId($eventId) {
    $this->event_id = $eventId;
  }
  
  public function fromArray($array) {
    return true;
  }
  
  public function fromPOSTArray($array, $groupIdx) {
    foreach ($this as $key => $value) {
      switch ($key) {
        case "validation":
        case "delete":
          break;
        
        case "event_option_group_id":
        case "event_id":
        case "group_type":
          if(isset($array[$key][$groupIdx]) && preg_match("~^[0-9]+$~", $array[$key][$groupIdx]))
            $this->$key = $array[$key][$groupIdx];
          else
            $this->$key = null;
          break;
        case "group_name":
        case "description":
          if(isset($array[$key][$groupIdx]))
            $this->$key = $array[$key][$groupIdx];
          else
            $this->$key = null;
          break;
        case "required":
        case "exclusive":
        case "allow_changes":
        case "enable_waitlist":
          $this->$key = ((isset($array[$key][$groupIdx]) && $array[$key][$groupIdx]) ? 1 : 0);
          break;
          
        case "eventOptions":
          foreach ($this->eventOptions as &$option) {
            if(!isset($array['event_option_id'][$groupIdx]) || 
               !in_array($option->event_option_id, $array['event_option_id'][$groupIdx]))
              $option->deleteOption(true);
          }
          if(isset($array['event_option'][$groupIdx])) {
            foreach ($array['event_option'][$groupIdx] as $optionIdx => $optionValue) {
              if($array['event_option_id'][$groupIdx][$optionIdx]) {
                $this->eventOptions[$array['event_option_id'][$groupIdx][$optionIdx]]->fromPOSTArray($array, $groupIdx, $optionIdx);
              }
              else {
                $newOption = new EventOption();
                $newOption->fromPOSTArray($array, $groupIdx, $optionIdx);
                $this->eventOptions[] = $newOption;
              }
            }
          }
          break;
        default:
          break;
      }
    }
    return true;
    //return $this->validation->validate($array);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - read
  public function readData(DbConnection $DB, $conditions) {
    $whereConditions = array();
    foreach ($conditions as $key => $value)
      if(array_key_exists($key, get_class_vars(get_class())))
        $whereConditions[] = "$key=$value";

    $result = $DB->query("SELECT * FROM event_option_groups WHERE ".implode(" AND ", $whereConditions).";");
    if(!$DB->errno() && $DB->num_rows($result)>0) {
      $group = $DB->fetch_array($result);
      foreach ($group as $property => $value) {
        if(property_exists($this, $property)) {
          $this->$property = $value;
        }
      }
      $result = $DB->query("SELECT event_option_id 
                            FROM event_options 
                            WHERE event_option_group_id='{$this->event_option_group_id}'
                            ORDER BY event_option_id;");
      $this->eventOptions = array();
      while ($optionId = $DB->fetch_array($result)) {
        $option = new EventOption();
        $option->readData($DB, array("event_option_id"=>$optionId['event_option_id']));
        $this->eventOptions[$optionId['event_option_id']] = $option;
      }
      
      return array(0, "");
    }
    else
      return array($DB->errno(), $DB->error());
  }
  
  static public function readList(DbConnection $DB) {
    return array();
  }
  
  static public function usingWaitlist($event_id) {
    global $DB;
    
    return $DB->result($DB->query("SELECT max(enable_waitlist) FROM event_option_groups WHERE event_id='{$event_id}';"), 0, 0);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - write
  public function writeData(DbConnection $DB) {
    if($this->delete)
      return self::deleteData($DB, $this->id);
    else if(isset($this->event_option_group_id) && $this->event_option_group_id) {
      $result = $DB->query("UPDATE event_option_groups SET
          event_id='{$this->event_id}',
          group_name='{$this->group_name}',
          group_type='{$this->group_type}',
          exclusive='".(in_array($this->group_type, array(OPTION_GROUP_FEES, OPTION_GROUP_TICKETS)) ? $this->exclusive : 0)."',
          allow_changes='".($this->group_type==OPTION_GROUP_ADDINFO ? $this->allow_changes : 0)."',
          enable_waitlist='".($_SESSION['event']->enable_waitlist && in_array($this->group_type, array(OPTION_GROUP_FEES)) ? $this->enable_waitlist : 0)."',
          required='{$this->required}',
          description='{$this->description}'
        WHERE event_option_group_id='{$this->event_option_group_id}';");
    }
    else {
      $result = $DB->query("INSERT INTO event_option_groups(
          event_id,
          group_name,
          group_type,
          exclusive,
          allow_changes,
          enable_waitlist,
          required,
          description
        ) VALUES(
          '{$this->event_id}',
          '{$this->group_name}',
          '{$this->group_type}',
          '".(in_array($this->group_type, array(OPTION_GROUP_FEES, OPTION_GROUP_TICKETS)) ? $this->exclusive : 0)."',
          '".($this->group_type==OPTION_GROUP_ADDINFO ? $this->allow_changes : 0)."',
          '".($_SESSION['event']->enable_waitlist && in_array($this->group_type, array(OPTION_GROUP_FEES)) ? $this->enable_waitlist : 0)."',
          '{$this->required}',
          '{$this->description}'
        );");
      
      $this->event_option_group_id = $DB->insert_id();
    }
    foreach ($this->eventOptions as &$option) {
      $option->setGroupId($this->event_option_group_id);
      $option->writeData($DB);
    }
    return array($DB->errno(), $DB->error());
  }

  static public function deleteData(DbConnection $DB, $id) {
    if(preg_match("~^[0-9]+$~", $id))
      $condition = "event_option_group_id='{$id}'";
    else
      $condition = $id;
      
    $groupIds = $DB->result($DB->query("SELECT group_concat(event_option_group_id SEPARATOR ', ') 
                                         FROM event_option_groups 
                                         WHERE $condition;"), 0, 0);
    $result = $DB->query("DELETE FROM event_option_groups WHERE {$condition};");
    
    if(!$DB->errno() && $DB->affected_rows()>0) {
      EventOption::deleteData($DB, "event_option_group_id IN ({$groupIds})");
      return array(0, "");
    }
    else 
      return array($DB->errno(), $DB->error());
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - misc
  static public function createTable(DbConnection $DB) {
    $result = $DB->query("
      CREATE TABLE event_option_groups (
        event_option_group_id int unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY,
        event_id int unsigned NOT NULL,
        group_name text,
        group_type tinyint unsigned,
        exclusive tinyint,
        allow_changes tinyint,
        enable_waitlist tinyint,
        required tinyint,
        description text,
        INDEX event_id (event_id)
      );");
    if ($result) { echo _("&nbsp;&nbsp;- creating table").": 'event_option_groups'.<br>\n"; }
    else { echo _("&nbsp;&nbsp;<b>- Error when creating table</b>")." 'event_option_groups'<br>\n"; $error = 1; }
  }  
}
?>