<?php 
require_once("icwfData.inc.php");
require_once("FormValidation.inc.php");

class PCGCharacter implements icwfData {
  private static $noExport = array();
  private static $notInArray = array("validation");
  private $validation;
  
  private $charakter_id;
  private $charakter_name;
  private $rasse_id;
  private $gottheit_id;
  private $stand_id;
  private $lp;
  private $mp;
  private $cp;
  private $sp;
  private $groesse;
  private $gewicht;
  private $haarfarbe;
  private $augenfarbe;
  private $beruf;
  private $kleidung;
  private $bes_kennzeichen;
  private $land_id;
  private $herkunft;
  private $hintergrund;
  private $verlauf;
  private $spiele;
  private $spiele_ep;
  private $vermoegen;
  private $spez_fertigkeiten;
  private $artefakte;
  private $charakter_bild;
  private $diebeszinken;
  private $ist_tot;
  
  private $player_familyname;
  private $player_givenname;
  private $ep_gesamt;
  private $ep_rest;
  private $f_auswahl;
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  private function initialize() {
    foreach ($this as $key => $value)
      if(in_array($key, array("verlauf", "spiele", "spiele_ep", "f_auswahl")))
        $this->$key = array();
      else if(in_array($key, array("mp", "sp", "cp")))
        $this->$key = "";
      else
        $this->$key = null;
  }

  public function __construct(FormValidation $validation=null) {
    $this->initialize();

    $this->gottheit_id = 1;
    $this->ep_gesamt = 0;
    $this->ep_rest = 0;
    $this->validation = $validation;
  }
  
  static function addMenuItems(CWFMainUI &$menu) {
    $menu->addMenuItem(array('pcg_hr'=>array('title'=>'', 'url'=>'', 'permission'=>'', 'submenu'=>''),
        'pcg'=>array('title'=>'Charakterverwaltung', 'url'=>'', 'permission'=>'', 'submenu'=>self::getMenuArray()
        )),
        array("Booking"));
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // get values
  public function __get($name) {
    if($name == "id") $name = "charakter_id";
    if (array_key_exists($name, get_class_vars(get_class())) && !in_array($name, self::$noExport))
      return $this->$name;

    return null;
  }

  public function getIDName() {
    return "charakter_id";
  }
  
  public function isNewChar() {
    return (count($this->spiele)==0);
  }
  
  public function hasMCSPoints($type, $fertigkeitId=null) {
    //Bardenmagie, Klerikalmagie, Schamanismus, Zauberei
    $mp = array(126, 127, 132, 135);
    //Alchemie, Pharmazie
    $cp = array(88, 106);
    //Schlösser öffnen
    $sp = array(109);
    
    if(!$fertigkeitId) {
      $hasMCSP = false;
      foreach ($$type as $Id)
        if(isset($this->f_auswahl[$Id]) && $this->f_auswahl[$Id]['stufe']>0) $hasMCSP = true;
      return $hasMCSP;
    }
    else
      return (in_array($fertigkeitId, $$type));
  }

  public function hasSkill($fertigkeitId) {
    return (isset($this->f_auswahl[$fertigkeitId]) && $this->f_auswahl[$fertigkeitId]['stufe']>0);
  }
  
  public function toArray() {
    $array = array();
    foreach ($this as $key => $value)
      if(!in_array($key, self::$notInArray))
      $array[$key] = $value;
    return $array;
  }

  public function getPostBackData() {
    return array($this->getIDName()=>$this->id);
  }

  private function getStufe() {
    $f = array();
    foreach($this->f_auswahl as $fertigkeitId => $fertigkeit)
      $f[$fertigkeitId] = $fertigkeit['stufe'];
    return $f;
  }
  
  public function getStand($standId=null) {
    $staende = array(
        array("name"=>"nicht von Stand", "ep"=>0, "geld"=>0),
        array("name"=>"Ritter", "ep"=>20,  "geld"=>100),
        array("name"=>"Baron",  "ep"=>70,  "geld"=>300),
        array("name"=>"Graf",   "ep"=>170, "geld"=>600),
        array("name"=>"Fürst",  "ep"=>270, "geld"=>1600),
        array("name"=>"Hezog",  "ep"=>370, "geld"=>3600),
        array("name"=>"Prinz",  "ep"=>520, "geld"=>6600)
    );
    if($standId===null) {
      if($this->isNewChar())
        return array_slice($staende, 0, 4);
      else
        return $staende;
    }
    else
      return $staende[$standId];
  }
  public function getStandEP($standId) {
    $stand = $this->getStand($standId);
    return $stand['ep'];
  }
  
  public function getXPforLP($lp, $lehrmethode, $fromLP=null) {
    if(!$fromLP) $fromLP = $this->lp;
    if($lp<=$fromLP) return 0;
    if($lp<5) return 0;
    if($lehrmethode==0) return 0;
    
    // Bardenmagie, Klerikalmagie, Schamanismus, Zauberei
    $ep = 0;
    if(count(array_intersect(array_keys($this->f_auswahl), array(126, 127, 132, 135)))>0) {
      for($i=$fromLP+1; $i<=$lp; $i++)
        if($i<19)
          $ep += floor(($i-3)/2);
        else
          $ep += floor((18-3)/2) + ($i-18);
    }
    else {
      for($i=$fromLP+1; $i<=$lp; $i++)
        if($i<29)
          $ep += floor(($i-1)/4);
        else
          $ep += floor((28-1)/4) +  floor(($i-29)/3) + 1;
    }
    
    //Schwächlich?
    if(isset($this->f_auswahl['12']) && $this->f_auswahl['12']['stufe']>0)
      $ep *= 2;
    
    return $ep;
  }
  
  public function checkPrerequisites($prereq) {
    if (!$prereq) return true;
    $f = $this->getStufe($this->f_auswahl);
    $g = $this->gottheit_id;
    return eval("return ({$prereq});");
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////////
  // set values
  private function importPCG1Char(DbConnection $DB) {
    $verlaufPCG1 = (is_array($this->verlauf) ? $this->verlauf : array());
    $this->verlauf = array();
    $this->verlauf[0] = array('id'=>'R', 'val'=>$this->rasse_id);
    $lp = $DB->result($DB->query("SELECT lp_start FROM pcg2_rassen WHERE rasse_id='{$this->rasse_id}';"), 0, 0);
    $mp = "";
    $cp = "";
    $sp = "";
    foreach ($verlaufPCG1 as $aktion) {
      $aktion = explode(">", $aktion);
      switch($aktion[0]) {
        case "LP":
          $ep = $this->getXPforLP($lp+$aktion[1], $aktion[2], $lp);
          $lp += $aktion[1];
          $this->verlauf[] = array("id"=>$aktion[0], "inc"=>$lp, "lm"=>$aktion[2], "ep"=>$ep, "id2"=>null);
          break;
        case "MP":
        case "CP":
        case "SP":
          $type = strtolower($aktion[0]);
          $$type += $aktion[1];
          $ep = $aktion[1]*$aktion[2];
          $this->verlauf[] = array("id"=>$aktion[0], "inc"=>$$type, "lm"=>$aktion[2], "ep"=>$ep, "id2"=>null);
          break;
        case "120":   //Spruchimmunität
        case "96":    //Giftimmunität
          $immun = $DB->fetch_array($DB->query("SELECT f.fertigkeit, fe.ep, k.kategorie ".
                                                    "FROM pcg2_fertigkeiten f ".
                                                    "  JOIN pcg2_fertigkeiten_ep fe ON (fe.fertigkeit_id=f.fertigkeit_id AND fe.rasse_id='{$this->rasse_id}') ".
                                                    "  JOIN pcg2_kategorien k ON (k.kategorie_id=f.kategorie_id) ".
                                                    "WHERE f.fertigkeit_id='{$aktion[0]}'"));
          $gegen = $DB->fetch_array($DB->query("SELECT f.fertigkeit_id, f.fertigkeit, fe.ep ".
                                                    "FROM pcg2_fertigkeiten f ".
                                                    "  JOIN pcg2_fertigkeiten_ep fe ON (fe.fertigkeit_id=f.fertigkeit_id AND fe.rasse_id='{$this->rasse_id}') ".
                                                    "  JOIN pcg2_kategorien k ON (k.kategorie_id=f.kategorie_id) ".
                                                    "WHERE f.fertigkeit_id='".abs($aktion[1])."'"));
          $ep = $immun['ep']*$gegen['ep']*$aktion[2];
          $this->verlauf[] = array("id"=>"i_{$gegen['fertigkeit_id']}", "inc"=>($aktion[1]>0 ? 1 : -1), "lm"=>$aktion[2], "ep"=>$ep, "id2"=>$aktion[0]);
          break;
        case "214":   // Sei verflucht
          if($aktion[1]>0)
            $stufe[214] += $aktion[1];
          if($stufe[214]==2)
            $aktion[0] = 372;
          else if($stufe[214]==3)
            $aktion[0] = 373;
          if($aktion[1]<0)
            $stufe[214] += $aktion[1];
        case "79":
        case "80":
        case "81":
        case "82":
        case "83":
        case "84":
        case "85":
          $mapping = array(79=>79, 80=>80, 81=>80, 82=>79, 83=>80, 84=>79, 85=>80);
          if(isset($mapping[$aktion[0]]))
            $aktion[0] = $mapping[$aktion[0]];
        default:
          if($aktion[1]<0) {
            $letzteAktion = end($this->verlauf);
            do {
              if($letzteAktion['id']==$aktion[0]) break;
            } while ($letzteAktion = prev($this->verlauf));
            $ep = $letzteAktion['ep'];
            unset($letzteAktion);
          }
          else
            $ep = $DB->result($DB->query("SELECT fe.ep ".
                                         "FROM pcg2_fertigkeiten f ".
                                         "  JOIN pcg2_fertigkeiten_ep fe ON (fe.fertigkeit_id=f.fertigkeit_id AND fe.rasse_id='{$this->rasse_id}') ".
                                         "  JOIN pcg2_kategorien k ON (k.kategorie_id=f.kategorie_id) ".
                                         "WHERE f.fertigkeit_id='{$aktion[0]}'"), 0, 0)
                 * $aktion[2];
          $this->verlauf[] = array("id"=>$aktion[0], "inc"=>$aktion[1], "lm"=>$aktion[2], "ep"=>$ep, "id2"=>null);

          $fertigkeitId = $aktion[0];
          $inc = $aktion[1];
          if($this->hasMCSPoints("mp", $fertigkeitId) && $inc>0 && $mp=="")
            $mp = $DB->result($DB->query("SELECT mp_start FROM pcg2_rassen WHERE rasse_id={$this->rasse_id};"), 0, 0);
          if($this->hasMCSPoints("cp", $fertigkeitId) && $inc>0 && $cp=="")
            $cp = 10;
          if($this->hasMCSPoints("sp", $fertigkeitId) && $inc>0 && $sp=="")
            $sp = 5;
          if($fertigkeitId==2)  //Ausgezehrt
            $lp -= $inc;
          break;
      }
    }
    unset($verlaufPCG1, $lp, $mp, $cp, $sp, $ep);
  }
  
  public function initNewChar(DbConnection $DB, $rasseId = null) {
    if($rasseId)
      $this->rasse_id = $rasseId;
    else {
      if(!is_array($this->verlauf[0]))  //PCG1-Charakter
        $this->importPCG1Char($DB);
      else
        $this->rasse_id = $this->verlauf[0]['val'];
    }
    
    $rasse = $DB->fetch_array($DB->query("SELECT * FROM pcg2_rassen WHERE rasse_id='{$this->rasse_id}';"));
    $this->lp = $rasse['lp_start'];
    $this->ep_gesamt = $rasse['xp_start'];
    $this->ep_rest = $rasse['xp_start'];
    $this->verlauf[0] = array('id'=>'R', 'val'=>$this->rasse_id);
    $this->f_auswahl = array();
    $this->mp = "";
    $this->cp = "";
    $this->sp = "";
    foreach ($this->spiele_ep as $ep) {
      $this->ep_gesamt += $ep;
      $this->ep_rest += $ep;
    }
    if(!$this->isNewChar() && $this->stand_id>0) $this->ep_rest -= $this->getStandEP($this->stand_id);
  }
  
  public function undoLastAction(DbConnection $DB) {
    if(count($this->verlauf)>1) {
      array_pop($this->verlauf);
      $this->initNewChar($DB);
      $this->verlaufProzessieren();
      return true;
    }
    
    return false;
  }
  
  public function __set($name, $value) {
    if($name == "id") $name = "charakter_id";
    if (array_key_exists($name, get_class_vars(get_class())) && !in_array($name, self::$noExport)) {
      switch($name) {
        case "ist_tot":
          $this->ist_tot = ($value ? 1 : 0);
          break;
        case "rasse_id":
          global $DB;
          $this->$name = $value;
          
          $rasse = $DB->fetch_array($DB->query("SELECT * FROM pcg2_rassen WHERE rasse_id='{$this->rasse_id}';"));
          $this->lp = $rasse['lp_start'];
          if($this->mp!="") $this->mp = $rasse['mp_start'];
          if($this->cp!="") $this->cp = 10;
          if($this->sp!="") $this->sp = 5;
          $this->verlauf[] = array('id'=>'R', 'val'=>$this->rasse_id, 'name'=>$rasse['rasse']);
          break;
        case "gottheit_id":
          if(preg_match('~^[0-9]+$~', $value)) $this->gottheit_id = $value;
        case "stand_id":
          if(!preg_match('~^[0-9]+$~', $value)) return;
          $alterStand = $this->stand_id;
          $this->stand_id = $value;
          if(!$this->isNewChar())
            $this->ep_rest -= $this->getStandEP($value) - $this->getStandEP($alterStand);
      }
    }
  }
  
  public function setSpiel($spielId, $text, $ep) {
    if(isset($this->spiele_ep[$spielId])) {
      $this->ep_gesamt -= $this->spiele_ep[$spielId];
      $this->ep_rest -= $this->spiele_ep[$spielId];
    }
    if(!$text && !$ep) {
      unset($this->spiele[$spielId]);
      unset($this->spiele_ep[$spielId]);
      $this->spiele = array_values($this->spiele);
      $this->spiele_ep = array_values($this->spiele_ep);
      if($this->isNewChar() && $this->stand_id) {
        $this->ep_rest += $this->getStandEP($this->stand_id);
        if($this->stand_id>3) $this->stand_id = 3;
      }
    }
    else {
      if($this->isNewChar() && $this->stand_id)
        $this->ep_rest -= $this->getStandEP($this->stand_id);
      $this->spiele[$spielId] = $text;
      $this->spiele_ep[$spielId] = $ep;
      $this->ep_gesamt += $ep;
      $this->ep_rest += $ep;
    }
  }

  private function setFAuswahl($f, $inc, $ep) {
    $this->f_auswahl[$f['fertigkeit_id']]['fertigkeit']       = $f['fertigkeit'];
    $this->f_auswahl[$f['fertigkeit_id']]['kategorie']        = $f['kategorie'];
    $this->f_auswahl[$f['fertigkeit_id']]['max_stufe']        = $f['max_stufe'];
    if(isset($f['immunitaet']))
      $this->f_auswahl[$f['fertigkeit_id']]['immunitaet']     = $f['immunitaet'];
    $stufeP = $this->f_auswahl[$f['fertigkeit_id']]['stufe'];
    $this->f_auswahl[$f['fertigkeit_id']]['stufe']           += $inc;
    $stufeC = $this->f_auswahl[$f['fertigkeit_id']]['stufe'];
    if($inc < 0)
      unset($this->f_auswahl[$f['fertigkeit_id']]['ep'][$stufeP]);
    else
      $this->f_auswahl[$f['fertigkeit_id']]['ep'][$stufeC]    = $ep;
    
    if($this->isNewChar() && $stufeC<=0) unset($this->f_auswahl[$f['fertigkeit_id']]);
  }
  
  //getSkillName: fertigkeit_id, fertigkeit, max_stufe, ep, stufe
  //getClass:    +kategorie, +voraussetzungen
  //Verlauf:      von LP auf LP oder EP
  //              bei MP/CP/SP: inc
  //Add Skill:    Nachteile: Kategorie, EP
  
  //return: $fertigkeit[fertigkeit, ep, maxStufe, kategorie], $lernmethode
  public function addSkill($fertigkeitId, $inc, $lernmethode, $aktualisiereVerlauf=true) {
    global $DB;
    if(!$this->rasse_id) return false;
    if(!preg_match("~^[0-9]+$~", $lernmethode)) return false;
    
    //LP
    if(strtoupper($fertigkeitId)=="LP") {
      if($this->isNewChar()) return false;
      if(!preg_match("~^-?[0-9]+$~", $inc)) return false;
      $maxLP = $DB->result($DB->query("SELECT lp_max FROM pcg2_rassen WHERE rasse_id='{$this->rasse_id}'"), 0, 0);
      if ($inc>$maxLP) return false;
      $ep = $this->getXPforLP($inc, $lernmethode);
      $this->lp = $inc;
      $this->ep_rest -= $ep;
      if ($aktualisiereVerlauf)
        $this->verlauf[] = array("id"=>"LP", "inc"=>$inc, "lm"=>$lernmethode, "ep"=>abs($ep));
      
      return array(array('fertigkeit_id'=>'LP', 'fertigkeit'=>'LP', 'ep'=>$ep, 'max_stufe'=>$maxLP), $lernmethode);
    }
    
    //MP, CP oder SP
    if(in_array(strtoupper($fertigkeitId), array('MP', 'CP', 'SP'))) {
      $fertigkeitId = strtolower($fertigkeitId);
      if($this->isNewChar()) return false;
      if(!$this->hasMCSPoints($fertigkeitId)) return false;
      if(!preg_match("~^-?[0-9]+$~", $inc)) return false;
      $ep = ($inc - $this->$fertigkeitId) * $lernmethode;
      if($ep<0) {
        $lernmethode = 1;
        $ep = 0;
      }
      $this->$fertigkeitId = $inc;
      $this->ep_rest -= $ep;
      if ($aktualisiereVerlauf)
        $this->verlauf[] = array("id"=>strtoupper($fertigkeitId), "inc"=>$inc, "lm"=>$lernmethode, "ep"=>abs($ep));
      
      return array(array('fertigkeit_id'=>$fertigkeitId, 'fertigkeit'=>strtoupper($fertigkeitId), 'ep'=>$ep, 'max_stufe'=>0), $lernmethode);
    }

    //Normale Fertigkeit
    if(in_array($fertigkeitId, array(96, 120))) {  //Giftimmunität oder Spruchimmunität
      $immun = $DB->fetch_array($DB->query("SELECT f.fertigkeit, fe.ep, k.kategorie ".
                                                "FROM pcg2_fertigkeiten f ".
                                                "  JOIN pcg2_fertigkeiten_ep fe ON (fe.fertigkeit_id=f.fertigkeit_id AND fe.rasse_id='{$this->rasse_id}') ".
                                                "  JOIN pcg2_kategorien k ON (k.kategorie_id=f.kategorie_id) ".
                                                "WHERE f.fertigkeit_id='{$fertigkeitId}'"));
      $gegen = $DB->fetch_array($DB->query("SELECT f.fertigkeit_id, f.fertigkeit, fe.ep ".
                                                "FROM pcg2_fertigkeiten f ".
                                                "  JOIN pcg2_fertigkeiten_ep fe ON (fe.fertigkeit_id=f.fertigkeit_id AND fe.rasse_id='{$this->rasse_id}') ".
                                                "  JOIN pcg2_kategorien k ON (k.kategorie_id=f.kategorie_id) ".
                                                "WHERE f.fertigkeit_id='{$inc}'"));
      $fertigkeit['fertigkeit_id'] = "i_".$gegen['fertigkeit_id'];
      $fertigkeit['fertigkeit'] = $immun['fertigkeit'] . " " . $gegen['fertigkeit'];
      $fertigkeit['ep'] = $immun['ep']*$gegen['ep'];
      $fertigkeit['voraussetzungen'] = "";
      $fertigkeit['voraussetzungen_text'] = "";
      $fertigkeit['max_stufe'] = 1;
      $fertigkeit['kategorie'] = $immun['kategorie'];
      $fertigkeit['immunitaet'] = $fertigkeitId;
      $fertigkeitId = $fertigkeit['fertigkeit_id'];
      $inc = 1;
      unset($immun);
      unset($gegen);
    }
    else if($inc<0 && preg_match("~^i_[0-9]+$~", $fertigkeitId)) {
      $fertigkeit['fertigkeit_id'] = $fertigkeitId;
      $fertigkeit['fertigkeit'] = $this->f_auswahl[$fertigkeitId]['fertigkeit'];
      $fertigkeit['ep'] = array_sum($this->f_auswahl[$fertigkeitId]['ep']);
      $fertigkeit['voraussetzungen'] = "";
      $fertigkeit['voraussetzungen_text'] = "";
      $fertigkeit['max_stufe'] = 1;
      $fertigkeit['kategorie'] = $this->f_auswahl[$fertigkeitId]['kategorie'];
      $fertigkeit['immunitaet'] = $this->f_auswahl[$fertigkeitId]['immunitaet'];
    }
    else {
      if(!preg_match("~^[0-9]+$~", $fertigkeitId)) return false;
      if(!preg_match("~^(-1|1)$~", $inc)) return false;
      $fertigkeit = $DB->fetch_array($DB->query("SELECT f.fertigkeit_id, f.fertigkeit, f.max_stufe, f.voraussetzungen, fe.ep, k.kategorie ".
                                                "FROM pcg2_fertigkeiten f ".
                                                "  JOIN pcg2_fertigkeiten_ep fe ON (fe.fertigkeit_id=f.fertigkeit_id AND fe.rasse_id='{$this->rasse_id}') ".
                                                "  JOIN pcg2_kategorien k ON (k.kategorie_id=f.kategorie_id) ".
                                                "WHERE f.fertigkeit_id='{$fertigkeitId}'"));
    }

    if($fertigkeit &&
       ($inc<0 ||
        ($fertigkeit['ep']>=0 &&
         $this->checkPrerequisites($fertigkeit['voraussetzungen']))) &&
       (!isset($this->f_auswahl[$fertigkeitId]) || 
        ($this->f_auswahl[$fertigkeitId]['stufe']+$inc<=$fertigkeit['max_stufe'] && $this->f_auswahl[$fertigkeitId]['stufe']+$inc>=0))) {
      //Lernmethode
      if(!$aktualisiereVerlauf)
        $lernmethode = $lernmethode;
      else if($fertigkeit['kategorie']=="Nachteil" && $inc>0) {
        if($this->isNewChar()) {
          $nachteilAnzahl = 0;
          $nachteilEP = 0;
          foreach($this->f_auswahl as $nachteil) {
            if($nachteil['kategorie']=="Nachteil") {
              $nachteilAnzahl++;
              $nachteilEP += array_sum($nachteil['ep']);
            } 
          }
          $lernmethode = (($nachteilAnzahl<3 && $nachteilEP<=15-$fertigkeit['ep']) ? 1 : 0);
        }
        else
          $lernmethode = 0;
      }
      else if ($inc < 0) {
        $aktion = end($this->verlauf);
        do {
          if($aktion['id']==$fertigkeitId) break;
        } while ($aktion = prev($this->verlauf));
        if ($fertigkeit['kategorie']=="Nachteil" && !$this->isNewChar())
          $lernmethode = 1;
        else
          $lernmethode = $aktion['lm'];
      }
      else
        $lernmethode = ($this->isNewChar() ? 1 : $lernmethode);
      //Ende Lernmethode
      
      //EP-Berechnung
      if($fertigkeit['kategorie']=="Nachteil") {
        if($inc>0 || !$aktualisiereVerlauf)
          $ep = -$fertigkeit['ep']*$inc*$lernmethode;
        else
          $ep = -$aktion['ep']*$inc*$lernmethode;
        if($this->isNewChar() || (!$aktualisiereVerlauf && $inc>0))
          $this->ep_gesamt -= $ep;
      }
      else {
        if($this->isNewChar() || $inc>0)
          $ep = $fertigkeit['ep']*$inc*$lernmethode;
        else
          $ep = 0;
      }
      $this->ep_rest -= $ep;
      //Ende EP-Berechnung
      
      $this->setFAuswahl($fertigkeit, $inc, abs($ep));
      
      //Besondere Aktionen
      if($this->hasMCSPoints("mp", $fertigkeitId) && $inc>0 && $this->mp=="")
        $this->mp = $DB->result($DB->query("SELECT mp_start FROM pcg2_rassen WHERE rasse_id={$this->rasse_id};"), 0, 0);
      if($this->hasMCSPoints("cp", $fertigkeitId) && $inc>0 && $this->cp=="")
        $this->cp = 10;
      if($this->hasMCSPoints("sp", $fertigkeitId) && $inc>0 && $this->sp=="")
        $this->sp = 5;
      if($fertigkeitId==2)  //Ausgezehrt
        $this->lp -= $inc;
      
      //Verlauf aktualisieren
      if ($aktualisiereVerlauf) {
        if($this->isNewChar() && $inc<0) {
          $aktion = end($this->verlauf);
          do
            if($aktion['id']==$fertigkeitId && $aktion['inc']>0) { unset($this->verlauf[key($this->verlauf)]); break; }
          while($aktion = prev($this->verlauf));
          $this->verlauf = array_values($this->verlauf);
          
          $this->initNewChar($DB);
          $this->verlaufProzessieren();
        }
        else
          $this->verlauf[] = array("id"=>$fertigkeitId, "inc"=>$inc, "lm"=>$lernmethode, "ep"=>($inc>0 ? abs($ep) : $aktion['ep']), "id2"=>(isset($fertigkeit['immunitaet']) ? $fertigkeit['immunitaet'] : null));
      }
      $fertigkeit['ep_soll'] = $fertigkeit['ep'];
      $fertigkeit['ep'] = abs($ep);
      return array($fertigkeit, $lernmethode);
    }
    return false;
  }
  
  public function verlaufProzessieren($recalcEP=false) {
    $verlauf = array_slice($this->verlauf, 1, 100, true);
    foreach ($verlauf as $verlaufId => $aktion) {
      switch ($aktion['id']) {
        case "R":
          $this->__set("rasse_id", $aktion['val']);
          $this->verlauf = array_slice($this->verlauf, 0, -1);
          break;
        default:
          if(preg_match('~i_~', $aktion['id'])) {
            $aktion['inc'] = substr($aktion['id'], 2);
            $aktion['id'] = $aktion['id2'];
          }
          if(list($fertigkeit, $lernmethode) = $this->addSkill($aktion['id'], $aktion['inc'], $aktion['lm'], false)) {
            if($recalcEP) $this->verlauf[$verlaufId]['ep'] = $fertigkeit['ep'];
          }
          else {
            unset($this->verlauf[$verlaufId]);
          }
      }
    }
    $this->verlauf = array_values($this->verlauf);
  }
  
  public function setValidation(FormValidation $validation) {
    $this->validation = $validation;
  }

  public function fromArray($array) {
    foreach ($this as $key => $value) {
      switch ($key) {
        case "validation":
        case "charakter_id":
        case "rasse_id":
        case "gottheit_id":
        case "stand_id":
        case "lp":
        case "mp":
        case "cp":
        case "sp":
        case "verlauf":
        case "spiele":
        case "spiele_ep":
        case "player_familyname":
        case "player_givenname":
        case "ep_gesamt":
        case "ep_rest":
        case "f_auswahl":
        case "charakter_bild":
        case "diebeszinken":
          break;
          
        default:
          if(isset($array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;
      }
    }
    return true;
    //return $this->validation->validate($array);
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - read
  public function readData(DbConnection $DB, $conditions) {
    if(is_array($conditions)) {
      $whereConditions = array();
      foreach ($conditions as $key => $value)
        if(array_key_exists($key, get_class_vars(get_class())))
          $whereConditions[] = "$key=$value";
      $conditions =  implode(" AND ", $whereConditions);
    }
    $result = $DB->query("SELECT pc.*, ".
                         "   (SELECT familyname FROM users u WHERE u.user_id=pc.user_id) player_familyname, ".
                         "   (SELECT givenname FROM users u WHERE u.user_id=pc.user_id) player_givenname ".
                         "FROM pcg_charaktere pc ".
                         "WHERE pc.deleted=0 ".
                         " AND ({$conditions});");
    if(!$DB->errno() && $DB->num_rows($result)>0) {
      $char = $DB->fetch_array($result);
      foreach ($char as $property => $value) {
        if(property_exists($this, $property)) {
          switch ($property) {
            case "verlauf":
            case "spiele":
            case "spiele_ep":
              eval('$this->$property = '.$value.';');
              break;
            default:
              $this->$property = $value;
              break;
          }
        }
      }

      return array(0, "");
    }
    else
      return array($DB->errno(), $DB->error());
  }

  static public function readList(DbConnection $DB) {
    $result = $DB->query("SELECT charakter_id id, charakter_name name, ist_tot FROM pcg_charaktere WHERE user_id='{$_SESSION['user']->user_id}' AND deleted=0 ORDER BY ist_tot ASC, charakter_name;");

    $list = array();
    while ($char = $DB->fetch_array($result))
      $list[] = $char;

    return $list;
  }
  
  static public function getMenuArray() {
    global $DB;

    $menuArray=array();
    $menuArray['pcg_new_char'] = array('title'=>'Neuer Charakter', 'url'=>'index.php/plugin/pcg2/PCGCharacterUI?charakter_id=0', 'permission'=>'', 'submenu'=>'');
    $menuArray["char_hr1"] = array('title'=>'', 'url'=>'', 'permission'=>'', 'submenu'=>'');
    if (isset($_SESSION['user'])) {
      $chars = static::readList($DB);
      $istTot = -1;
      foreach ($chars as $char) {
        if($istTot>=0 && $char['ist_tot'])
          $menuArray["char_hr2"] = array('title'=>'', 'url'=>'', 'permission'=>'', 'submenu'=>'');
        $menuArray["char_{$char['id']}"] = array('title'=>$char['name'], 'url'=>'index.php/plugin/pcg2/PCGCharacterUI?charakter_id='.$char['id'], 'permission'=>'', 'submenu'=>'');
        $istTot = $char['ist_tot'];
      }
    }
    return $menuArray;
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - write
  public function writeData(DbConnection $DB) {
    if ($_POST['pcg_submit'] == "Bild hochladen" || $_POST['pcg_submit'] == "Zinken hochladen") {
      if(!$this->charakter_id) {
        $tmp = $_POST['pcg_submit'];
        unset($_POST['pcg_submit']); 
        $this->writeData($DB);
        $_POST['pcg_submit'] = $tmp;
      }
      switch ($_POST['pcg_submit']) {
        case "Bild hochladen":
          $datei_feld = "datei_bild";
          $db_feld = "charakter_bild";
          $fn_prefix = "b_";
          $size_max = array(150, 200);
          break;
        case "Zinken hochladen":
          $datei_feld = "datei_zinken";
          $db_feld = "diebeszinken";
          $fn_prefix = "z_";
          $size_max = array(200, 200);
          break;
      }
      if ($_FILES[$datei_feld]['name'] && ($_FILES[$datei_feld]['size']>0)) {
        $size=getimagesize($_FILES[$datei_feld]['tmp_name']);
        
        if ($size[2]==2) { // $size[2]==2 => image type=JPEG
          $old_file = $DB->result($DB->query("SELECT {$db_feld} FROM pcg_charaktere WHERE charakter_id='{$this->charakter_id}' AND user_id='{$_SESSION['user']->user_id}';"), 0, 0);
          if ($old_file) {
            $success = unlink(PCG_PATH."upload/{$old_file}");
            $file_name = $old_file;
          }
          else {
            do {
              $file_name = $fn_prefix . CWFTools::generateFileName() . "." . substr(strrchr($_FILES[$datei_feld]['name'], "."), 1);
            } while ($DB->num_rows($DB->query("SELECT charakter_id FROM pcg_charaktere WHERE {$db_feld}='{$file_name}';")));
          }
          
          move_uploaded_file($_FILES[$datei_feld]['tmp_name'], PCG_PATH."upload/{$file_name}");
            
          if ($size[0]>$size_max[0] || $size[1]>$size_max[1]) {    // width > 150 or height > 200
            $source = imagecreatefromjpeg(PCG_PATH."upload/{$file_name}");
  
            $a = $size[0]/$size_max[0];
            $b = $size[1]/$size_max[1];
            if ($a>$b) {
              $target  = imagecreatetruecolor($size_max[0], $size[1]/$a);
              imagecopyresampled($target, $source, 0, 0, 0, 0, $size_max[0], $size[1]/$a, $size[0], $size[1]);
            }
            else {
              $target  = imagecreatetruecolor($size[0]/$b, $size_max[1]);
              imagecopyresampled($target, $source, 0, 0, 0, 0, $size[0]/$b, $size_max[1], $size[0], $size[1]);
            }
            
            imagejpeg($target, PCG_PATH."upload/{$file_name}");
          }
  
          $this->$db_feld = $file_name;
    
          $DB->query("UPDATE pcg_charaktere
                      SET {$db_feld}='{$file_name}'
                      WHERE charakter_id='{$this->charakter_id}'
                        AND user_id='{$_SESSION['user']->user_id}';");
        }
      }
    }
    else {
      $this->spiele = array_values($this->spiele);
      $this->spiele_ep = array_values($this->spiele_ep);
  
      if (!isset($this->charakter_name) || $this->charakter_name=="")
        $this->charakter_name = "Kein Name";
        
      if ($this->charakter_id>0) {
        $DB->query("UPDATE pcg_charaktere
                    SET charakter_name='{$this->charakter_name}',
                      rasse_id='{$this->rasse_id}',
                      gottheit_id='{$this->gottheit_id}',
                      stand_id='{$this->stand_id}',
                      lp='{$this->lp}',
                      mp=".($this->mp=="" ? "null" : "'{$this->mp}'").",
                      cp=".($this->cp=="" ? "null" : "'{$this->cp}'").",
                      sp=".($this->sp=="" ? "null" : "'{$this->sp}'").",
                      groesse=".($this->groesse=="" ? "null" : "'{$this->groesse}'").",
                      gewicht=".($this->gewicht=="" ? "null" : "'{$this->gewicht}'").",
                      haarfarbe='{$this->haarfarbe}',
                      augenfarbe='{$this->augenfarbe}',
                      beruf='{$this->beruf}',
                      kleidung='{$this->kleidung}',
                      bes_kennzeichen='{$this->bes_kennzeichen}',
                      land_id='{$this->land_id}',
                      herkunft='{$this->herkunft}',
                      hintergrund='{$this->hintergrund}',
                      verlauf='".str_replace("'", "\'", var_export($this->verlauf, true))."',
                      spiele='".str_replace("'", "\'", var_export($this->spiele, true))."',
                      spiele_ep='".str_replace("'", "\'", var_export($this->spiele_ep, true))."',
                      vermoegen='{$this->vermoegen}',
                      spez_fertigkeiten='{$this->spez_fertigkeiten}',
                      artefakte='{$this->artefakte}'
                  WHERE charakter_id='{$this->charakter_id}'
                    AND user_id='{$_SESSION['user']->user_id}';");
      }
      else {
        $DB->query("INSERT INTO pcg_charaktere
                    (charakter_name, user_id, rasse_id, gottheit_id, stand_id, lp, mp, cp, sp,
                     groesse, gewicht, haarfarbe, augenfarbe, beruf, kleidung,
                     bes_kennzeichen, land_id, herkunft, hintergrund, verlauf, 
                     spiele, spiele_ep, vermoegen, spez_fertigkeiten, artefakte, 
                     ist_tot, deleted) 
                  VALUES (
                    '{$this->charakter_name}',
                    '{$_SESSION['user']->user_id}',
                    '{$this->rasse_id}',
                    '{$this->gottheit_id}',
                    '{$this->stand_id}',
                    '{$this->lp}',
                    ".($this->mp=="" ? "null" : "'{$this->mp}'").",
                    ".($this->cp=="" ? "null" : "'{$this->cp}'").",
                    ".($this->sp=="" ? "null" : "'{$this->sp}'").",
                    ".($this->groesse=="" ? "null" : "'{$this->groesse}'").",
                    ".($this->gewicht=="" ? "null" : "'{$this->gewicht}'").",
                    '{$this->haarfarbe}',
                    '{$this->augenfarbe}',
                    '{$this->beruf}',
                    '{$this->kleidung}',
                    '{$this->bes_kennzeichen}',
                    '{$this->land_id}',
                    '{$this->herkunft}',
                    '{$this->hintergrund}',
                    '".str_replace("'", "\'", var_export($this->verlauf, true))."',
                    '".str_replace("'", "\'", var_export($this->spiele, true))."',
                    '".str_replace("'", "\'", var_export($this->spiele_ep, true))."',
                    '{$this->vermoegen}',
                    '{$this->spez_fertigkeiten}',
                    '{$this->artefakte}',
                    0,
                    0
                  );");
        $this->charakter_id=$DB->insert_id();
      }
   
      $DB->query("DELETE FROM pcg_char_fertigkeiten WHERE charakter_id={$this->charakter_id};");
      if (count($this->f_auswahl)>0) {
        $str = "INSERT INTO pcg_char_fertigkeiten (charakter_id, fertigkeit_id, stufe, immunitaet_id) VALUES \n";
        foreach ($this->f_auswahl as $fertigkeitId => $fertigkeit) {
          if(preg_match("~^i_([0-9]+)$~", $fertigkeitId, $match)) {
            $str .= "  ({$this->charakter_id}, {$fertigkeit['immunitaet']}, {$fertigkeit['stufe']}, {$match[1]}), \n";
          }
          //if(is_array($fertigkeit)) {
          //  foreach($fertigkeit as $id2 => $stufe2)
          //    $str .= "  ({$this->charakter_id}, {$fertigkeitId}, {$stufe2}, $id2), \n";
          //}
          //else 
          else if ($fertigkeitId!=0 && $fertigkeit>0) {
            $str .= "  ({$this->charakter_id}, {$fertigkeitId}, {$fertigkeit['stufe']}, null), \n";
          }
        }
        $str = substr_replace($str, ";", -3);
        $DB->query($str);
      }
    }

    return array($DB->errno(), $DB->error());
  }

  static public function deleteData(DbConnection $DB, $id, $setDead=false) {
    if(!preg_match("~^[0-9]+$~", $id))
      return array(1, "ID not valid.");
  
    if($setDead)
      $result = $DB->query("UPDATE pcg_charaktere SET ist_tot=abs(ist_tot - 1) WHERE charakter_id='{$id}';");
    else
      $result = $DB->query("UPDATE pcg_charaktere SET deleted=1 WHERE charakter_id='{$id}';");
  
    if(!$DB->errno() && $DB->affected_rows()>0)
      return array(0, "");
    else
      return array($DB->errno(), $DB->error());
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - misc
  static public function createTable(DbConnection $DB) {
/*

CREATE TABLE IF NOT EXISTS pcg_charaktere (
  charakter_id int unsigned NOT NULL auto_increment,
  user_id int unsigned NOT NULL,
  charakter_name text NOT NULL,
  rasse_id tinyint unsigned,
  gottheit_id tinyint unsigned,
  lp tinyint unsigned,
  mp tinyint unsigned default NULL,
  cp tinyint unsigned default NULL,
  sp tinyint unsigned default NULL,
  groesse smallint unsigned,
  gewicht smallint unsigned,
  haarfarbe text,
  augenfarbe text,
  beruf text,
  kleidung text,
  bes_kennzeichen text,
  land_id int unsigned NOT NULL,
  herkunft text NOT NULL,
  hintergrund text NOT NULL,
  verlauf text,
  spiele text,
  spiele_ep text,
  vermoegen smallint,
  spez_fertigkeiten text,
  artefakte text,
  charakter_bild text,
  diebeszinken text,
  deleted tinyint,
  PRIMARY KEY (charakter_id)
);

CREATE TABLE IF NOT EXISTS pcg_char_fertigkeiten (
  charakter_id int unsigned NOT NULL,
  fertigkeit_id int unsigned NOT NULL,
  stufe tinyint unsigned NOT NULL,
  immunitaet_id int unsigned,
  PRIMARY KEY (charakter_id, fertigkeit_id)
);

CREATE TABLE IF NOT EXISTS pcg_laender (
  land_id int unsigned NOT NULL AUTO_INCREMENT,
  land_name text NOT NULL,
  PRIMARY KEY (land_id)
);

--
-- Dumping data for table phoenix_pcg_laender
--

INSERT INTO pcg_laender (land_id, land_name) VALUES
(1, 'Anguir'),
(2, 'Verilion'),
(3, 'Hadran'),
(4, 'Aklon'),
(5, 'Murash'),
(6, 'Rulos'),
(7, 'Cahirsheveen'),
(8, 'Haralin'),
(9, 'Helingard'),
(10, 'Kolte'),
(11, 'Lir'),
(12, 'Orklande'),
(13, 'Raikal'),
(14, 'Schwarzburg'),
(15, 'Shai Anarat'),
(16, 'Sternthal'),
(17, 'Sythia'),
(18, 'Urs Sanktum'),
(19, 'Wilde Lande'),
(20, 'Aklon - Stadt Tulderon'),
(22, 'Sheldiria'),
(25, 'Borbano'),
(26, 'Falkenstein');

*/
  }

  ////////////////////////////////////////////////////////////////////////////////
  // Ausgewählte Fertigkeiten prozessieren
  //

}
?>
