<?php
require_once("cwfUI.inc.php");
require_once(PCG_PATH."PCGSelectChar.inc.php");

class PCGSelectCharUI extends cwfUI {
  protected static $accessRight = "";
  protected static $dataClass = "PCGSelectChar";
  public $data;
  protected $error;
  
  public function __construct(DbConnection $DB) {
    parent::__construct();

    $this->data = new PCGSelectChar(self::getValidationRules());
    
    if(isset($_SESSION['booking_id']) && $_SESSION['booking_id'])
      $this->data->readData($DB, array("booking_id"=>$_SESSION['booking_id']));
    
    $this->title = _("Registration");
    $this->header = "  <script type=\"text/javascript\">
    $(document).ready(function(){
      $(\"form\").validate({
        errorClass: 'invalid',\n".
        $this->data->validation->getValidationRulesJQuery("        ")."
      });
    });
  </script>\n";
    
    $this->error="";
    if((isset($_POST['kdb_save']) && $_POST['kdb_save'])) {
      $this->saveForm($DB);
    }
  }
  
  public function saveForm(DbConnection $DB) {
    if($this->data->fromArray($_POST)) {
      list($errno, $error) = $this->data->writeData($DB);
      if ($errno) {
        $this->error = $error;
        return 0;
      }
      return 1;
    }
    return 0;
  }
  
  static public function getValidationRules() {
    $userProperties = explode(",", KDbConfig::getUserProperties());
    $userPropertiesReq = explode(",", KDbConfig::getUserPropertiesReq());
    
    $rules = array(
        "end_char_view"=>array("date_".getenv("LC_ALL")=>true)
    );
    $messages = array(
        "end_char_view"=>array("date_".getenv("LC_ALL")=>_("Please enter a valid date, e.g. ").CWFTools::dateISO2Local("2000-12-31", getenv("LC_ALL")))
    );
    $validationRules = new FormValidation($rules, $messages);
    return $validationRules;
  }
  
  public function getErrorMessage() {
    return $this->error;
  }
  
  public function printSubmitButtons() {
    return 1;
  }
  
  public function getHTML(DbConnection $DB) {
    $this->getHTMLRegistration($DB);
  }

  public function getHTMLRegistration(DbConnection $DB) {
?>
<h3>Charakter-Auswahl</h3>

<p>
  <label for="charakter_id">Charakter:</label>
  <select id="charakter_id" name="charakter_id" size="1">
    <option value="0">Bitte Charakter auswählen</option>
<?php
    $charaktere = PCGCharacter::readList($DB);
    $istTot=-1;
    $totString = array('Aktiv', 'Tot');
    foreach ($charaktere as $charakter) {
      if($charakter['ist_tot']!=$istTot)
        echo ($istTot>=0 ? "    </optgroup>\n" : "")."    <optgroup label=\"{$totString[$charakter['ist_tot']]}\">\n";
      if ($charakter['id']==$this->data->charakter_id)
        echo "      <option value={$charakter['id']} selected>{$charakter['name']}</option>\n";
      else
        echo "      <option value={$charakter['id']}>{$charakter['name']}</option>\n";
      $istTot = $charakter['ist_tot'];
    }
?>
    </optgroup>
  </select>
</p>

<p>
  <label for="end_char_view">Charakter freigeben bis:</label>
  <input type="text" id="end_char_view" name="end_char_view" value="<?=CWFTools::dateISO2Local($this->data->end_char_view, getenv("LC_ALL"))?>" placeholder="<?=CWFTools::dateISO2Local("2000-12-31", getenv("LC_ALL"))?>">
  <span class="form_additional_info">Veranstaltungszeitraum: <?=$_SESSION['event']->event_period;?></span>
</p>
<?php
  }
  
  private function getClassString($classArray) {
    if(count($classArray)>0)
      return "class=\"".implode(" ", $classArray)."\" ";
    else
      return "";
  }
}

?>